@@ -29,13 +29,13 @@ class Config(object):
 
     # udocker installation tarball the release is the minimum requirement
     # the actual tarball used in the installation can have a higher version
-    conf['tarball_release'] = "1.2.8"
+    conf['tarball_release'] = "1.2.10"
     conf['tarball'] = (
         "https://download.ncg.ingrid.pt/"
-        "webdav/udocker/udocker-englib-1.2.8.tar.gz"
+        "webdav/udocker/udocker-englib-1.2.10.tar.gz"
         " "
         "https://raw.githubusercontent.com"
-        "/jorge-lip/udocker-builds/master/tarballs/udocker-englib-1.2.8.tar.gz"
+        "/jorge-lip/udocker-builds/master/tarballs/udocker-englib-1.2.10.tar.gz"
     )
     conf['installinfo'] = [
         "https://raw.githubusercontent.com/indigo-dc/udocker/master/messages", ]
@@ -46,7 +46,7 @@ class Config(object):
     conf['tmpdir'] = os.getenv("TMPDIR", "/tmp")    # for tmp files only
 
     # defaults for container execution
-    conf['cmd'] = ["/bin/bash", "-i"]  # Comand to execute
+    conf['cmd'] = ["bash", "-i"]  # Comand to execute
 
     # default path for executables
     conf['root_path'] = "/usr/sbin:/sbin:/usr/bin:/bin"
@@ -57,7 +57,7 @@ class Config(object):
                             "/etc/resolv.conf", "/etc/host.conf",
                             "/lib/modules", )
 
-    # POSSIBLE DEPRECATED
+    # DEPRECATED
     # directories to be mapped in containers with: run --hostauth
     # conf['hostauth_list'] = ("/etc/passwd", "/etc/group",
     #                               "/etc/shadow", "/etc/gshadow", )
@@ -71,7 +71,10 @@ class Config(object):
 
     # container execution mode if not set via setup
     # Change it to P2 if execution problems occur
-    conf['default_execution_mode'] = "P1"
+    conf['override_default_execution_mode'] = ""
+    conf['default_execution_modes'] = {'x86_64': "P1", 'x86': "P1",
+                                       'arm64': "P1", 'arm': "P2",
+                                       'ppc64le': "R1", 'DEFAULT': "R1"}
 
     # PRoot override seccomp
     # conf['proot_noseccomp'] = True
@@ -85,12 +88,19 @@ class Config(object):
     # conf['fakechroot_so'] = "libfakechroot-CentOS-7-x86_64.so"
     conf['fakechroot_so'] = None
 
-    # translate symbolic links in pathnames None means automatic
+    # translate symbolic links into pathnames None means automatic
     conf['fakechroot_expand_symlinks'] = None
 
+    # patterns to search for libc.so for bypass in fakechroot
+    conf['libc_search'] = ("/lib64/libc.so.[0-9]", "/usr/lib64/libc.so.[0-9]",
+                           "/usr/lib/libc.so.[0-9]", "/lib/libc.so.[0-9]",
+                           "/usr/libc.so.[0-9]", "/libc.so.[0-9]", "/libc.so",)
+
+    # override the above search for libc with a specified relative pathname
+    conf['fakechroot_libc'] = None
+
     # sharable library directories
-    conf['lib_dirs_list_x86_64'] = ("/usr/lib/x86_64-linux-gnu",
-                                    "/usr/lib64",)
+    conf['lib_dirs_list_nvidia'] = ("/usr/lib/x86_64-linux-gnu", "/usr/lib64",)
 
     # fakechroot sharable library directories
     conf['lib_dirs_list_essential'] = ("/lib/x86_64-linux-gnu",
@@ -110,6 +120,7 @@ class Config(object):
     # Force the use of specific executables
     # UDOCKER = use executable from the udocker binary distribution/tarball
     conf['use_proot_executable'] = "UDOCKER"
+    conf['use_patchelf_executable'] = "UDOCKER"
     conf['use_runc_executable'] = ""
     conf['use_singularity_executable'] = ""
 
@@ -232,11 +243,13 @@ class Config(object):
             os.getenv("UDOCKER_REGISTRY", Config.conf['dockerio_registry_url'])
         Config.conf['tarball'] = \
             os.getenv("UDOCKER_TARBALL", Config.conf['tarball'])
-        Config.conf['default_execution_mode'] = \
+        Config.conf['override_default_execution_mode'] = \
             os.getenv("UDOCKER_DEFAULT_EXECUTION_MODE",
-                      Config.conf['default_execution_mode'])
+                      Config.conf['override_default_execution_mode'])
         Config.conf['fakechroot_so'] = \
             os.getenv("UDOCKER_FAKECHROOT_SO", Config.conf['fakechroot_so'])
+        Config.conf['fakechroot_libc'] = \
+            os.getenv("UDOCKER_FAKECHROOT_LIBC", Config.conf['fakechroot_libc'])
         Config.conf['tmpdir'] = os.getenv("UDOCKER_TMP", Config.conf['tmpdir'])
         Config.conf['keystore'] = \
             os.getenv("UDOCKER_KEYSTORE", Config.conf['keystore'])
@@ -252,6 +265,9 @@ class Config(object):
         Config.conf['use_singularity_executable'] = \
             os.getenv("UDOCKER_USE_SINGULARITY_EXECUTABLE",
                       Config.conf['use_singularity_executable'])
+        Config.conf['use_patchelf_executable'] = \
+            os.getenv("UDOCKER_USE_PATCHELF_EXECUTABLE",
+                      Config.conf['use_patchelf_executable'])
 
         Config.conf['fakechroot_expand_symlinks'] = \
             os.getenv("UDOCKER_FAKECHROOT_EXPAND_SYMLINKS",
