@@ -93,12 +93,12 @@ class NvidiaMode(object):
         Msg().out("Debug: List nvidia libs", lib_list, l=Msg.DBG)
         return lib_list
 
-    def _find_host_dir_ldconfig(self, arch="x86-64"):
+    def _find_host_dir_ldconfig(self, arch=""):
         """Find host nvidia libraries via ldconfig"""
         dir_list = set()
         ld_data = Uprocess().get_output(["ldconfig", "-p"])
         if ld_data:
-            regexp = "[ |\t]%s[^ ]* .*%s.*=> (/.*)"
+            regexp = "[ |\t]%s[^ ]* .*%s => (/.*)"
             for line in ld_data.split('\n'):
                 for lib in self._nvidia_main_libs:
                     match = re.search(regexp % (lib, arch), line)
@@ -122,7 +122,7 @@ class NvidiaMode(object):
     def _find_host_dir(self):
         """Find the location of the host nvidia libraries"""
         dir_list = set()
-        library_path = ':'.join(Config.conf['lib_dirs_list_x86_64'])
+        library_path = ':'.join(Config.conf['lib_dirs_list_nvidia'])
         dir_list.update(self._find_host_dir_ldpath(library_path))
         dir_list.update(self._find_host_dir_ldconfig())
         library_path = os.getenv("LD_LIBRARY_PATH", "")
