@@ -4,9 +4,9 @@
 import os
 
 from udocker.msg import Msg
+from udocker.helper.unique import Unique
 from udocker.commonlocalfile import CommonLocalFileApi
 from udocker.utils.fileutil import FileUtil
-from udocker.helper.unique import Unique
 
 
 class OciLocalFileAPI(CommonLocalFileApi):
@@ -21,12 +21,11 @@ class OciLocalFileAPI(CommonLocalFileApi):
         structure["repolayers"] = {}
         structure["manifest"] = {}
         f_path = tmp_imagedir + '/'
-        structure["oci-layout"] = \
-                self.localrepo.load_json(f_path + "oci-layout")
-        structure["index"] = \
-                self.localrepo.load_json(f_path + "index.json")
+        structure["oci-layout"] = self.localrepo.load_json(f_path + "oci-layout")
+        structure["index"] = self.localrepo.load_json(f_path + "index.json")
         if not (structure["index"] and structure["oci-layout"]):
             return {}
+
         for fname in os.listdir(tmp_imagedir + "/blobs"):
             f_path = tmp_imagedir + "/blobs/" + fname
             if FileUtil(f_path).isdir():
@@ -34,19 +33,16 @@ class OciLocalFileAPI(CommonLocalFileApi):
                 for layer_f in os.listdir(f_path):
                     layer_id = layer_algorithm + ':' + layer_f
                     structure["repolayers"][layer_id] = {}
-                    structure["repolayers"][layer_id]["layer_f"] = \
-                            f_path + '/' + layer_f
-                    structure["repolayers"][layer_id]["layer_a"] = \
-                            layer_algorithm
-                    structure["repolayers"][layer_id]["layer_h"] = \
-                            layer_f
+                    structure["repolayers"][layer_id]["layer_f"] = f_path + '/' + layer_f
+                    structure["repolayers"][layer_id]["layer_a"] = layer_algorithm
+                    structure["repolayers"][layer_id]["layer_h"] = layer_f
+
         return structure
 
     def _get_from_manifest(self, structure, imagetag):
         """Search for OCI manifest and return Config and layer ids"""
         try:
-            config_layer = \
-                    structure["manifest"][imagetag]["json"]["config"]["digest"]
+            config_layer = structure["manifest"][imagetag]["json"]["config"]["digest"]
         except (KeyError, ValueError, TypeError):
             config_layer = ""
         try:
@@ -73,22 +69,19 @@ class OciLocalFileAPI(CommonLocalFileApi):
         imagetag = imagerepo + ':' + tag
         structure["manifest"][imagetag] = {}
         structure["manifest"][imagetag]["json"] = \
-                self.localrepo.load_json(structure["repolayers"]\
-                [manifest["digest"]]["layer_f"])
+            self.localrepo.load_json(structure["repolayers"][manifest["digest"]]["layer_f"])
         structure["manifest"][imagetag]["json_f"] = \
-                structure["repolayers"][manifest["digest"]]["layer_f"]
+            structure["repolayers"][manifest["digest"]]["layer_f"]
         return self._load_image(structure, imagerepo, tag)
 
     def _load_repositories(self, structure):
         """Load OCI image repositories"""
         loaded_repositories = []
         for manifest in structure["index"]["manifests"]:
-            if manifest["mediaType"] == \
-                    "application/vnd.oci.image.manifest.v1+json":
+            if manifest["mediaType"] == "application/vnd.oci.image.manifest.v1+json":
                 loaded_repositories.append(
                     self._load_manifest(structure, manifest))
-            elif manifest["mediaType"] == \
-                    "application/vnd.oci.image.index.v1+json":
+            elif manifest["mediaType"] == "application/vnd.oci.image.index.v1+json":
                 loaded_repositories.extend(self._load_repositories(
                     self.localrepo.load_json(
                         structure["repolayers"]
@@ -98,12 +91,10 @@ class OciLocalFileAPI(CommonLocalFileApi):
     def _load_image_step2(self, structure, imagerepo, tag):
         """Prepare load of OCI image"""
         imagetag = imagerepo + ':' + tag
-        (config_layer_id, layers) = \
-                self._get_from_manifest(structure, imagetag)
+        (config_layer_id, layers) = self._get_from_manifest(structure, imagetag)
         if config_layer_id:
             json_file = structure["repolayers"][config_layer_id]["layer_f"]
-            self._move_layer_to_v1repo(json_file, config_layer_id,
-                                       "container.json")
+            self._move_layer_to_v1repo(json_file, config_layer_id, "container.json")
         layer_hash_list = []
         for layer_id in layers:
             Msg().out("Info: adding layer:", layer_id, l=Msg.INF)
