@@ -171,6 +171,23 @@ class UdockerTools(object):
         FileUtil(tmpdir).remove(recursive=True)
         return (status, tarball_version)
 
+    def _clean_install(self, tfile):
+        """Remove files before install"""
+        for tar_in in tfile.getmembers():
+            basename = os.path.basename(tar_in.name)
+            if tar_in.name.startswith("udocker_dir/bin/"):
+                f_path = self.localrepo.bindir + '/' + basename
+                FileUtil(f_path).register_prefix()
+                FileUtil(f_path).remove(recursive=True)
+            if tar_in.name.startswith("udocker_dir/lib/"):
+                f_path = self.localrepo.libdir + '/' + basename
+                FileUtil(f_path).register_prefix()
+                FileUtil(f_path).remove(recursive=True)
+            if tar_in.name.startswith("udocker_dir/doc/"):
+                f_path = self.localrepo.docdir + '/' + basename
+                FileUtil(f_path).register_prefix()
+                FileUtil(f_path).remove(recursive=True)
+
     def _install(self, tarball_file):
         """Install the tarball"""
         if not (tarball_file and os.path.isfile(tarball_file)):
@@ -181,6 +198,7 @@ class UdockerTools(object):
         try:
             tfile = tarfile.open(tarball_file, "r:gz")
             FileUtil(self.localrepo.bindir).rchmod()
+            self._clean_install(tfile)
             for tar_in in tfile.getmembers():
                 if tar_in.name.startswith("udocker_dir/bin/"):
                     tar_in.name = os.path.basename(tar_in.name)
@@ -258,6 +276,7 @@ class UdockerTools(object):
     def install(self, force=False):
         """Get the udocker tools tarball and install the binaries"""
         if self.is_available() and not force:
+            Msg().out("Debug: already installed, installation skipped", l=Msg.DBG)
             return True
 
         if not self._autoinstall and not force:
@@ -271,7 +290,7 @@ class UdockerTools(object):
             return True
 
         Msg().out("Info: udocker command line interface", __version__)
-        Msg().out("Info: searching for udockertools", self._tarball_release, l=Msg.INF)
+        Msg().out("Info: searching for udockertools >=", self._tarball_release, l=Msg.INF)
         retry = self._installretry
         while retry:
             if self._install_logic(force):
